//
//	odfinfo: give info about ODF
//
#include <iostream>
#include <map>

#include "OdfInfo.h"
#include "Xmm.h"
#include "ModeType.h"
#include <SasVersion.h>
#include "SasFileLocator.h"     //      for locating instrument config files
#include <stringutil.h>

#include "odfParamCreator.h"
#include <qdom.h>
#include <qfile.h>
#include <fstream>
#include "StringFns.h"

using namespace std;

void odfParamCreator()
{

	string outputFileName =  stringParameter("outputFileName");

  const OdfInfo::Odf *odf = OdfInfo::server->odf();
  errstr<<"Observation start: "<<odf->start()<< Sas::message(Msg::App, Msg::Verbose);
  errstr<<"Observation end  : "<<odf->stop()<<Sas::message(Msg::App, Msg::Verbose);
  errstr<<"Revolution number: "<<odf->revolution()<<Sas::message(Msg::App, Msg::Verbose);
  errstr<<"PI               : "<<odf->proposalData().GoName()<<Sas::message(Msg::App, Msg::Verbose);

  QDomDocument doc("BODY");
  QDomElement root = doc.createElement("BODY");
  doc.appendChild(root);

  QDomElement tag = doc.createElement("CONFIG");
  string sasv = SasVersion::sas_release;
  tag.setAttribute("SASVersion",sasv.c_str());
  root.appendChild(tag);


  /*
   * Check if all the instruments have at least one exposure
   */
  // Hash map containing exposure flag for each instrument
  map <string,string> expoMap;

  for (unsigned i=0; i<xmmPayLoad.instruments().size(); ++i) {
    XmmInstrument *ins = xmmPayLoad.instruments()[i];

    if (ins->id() == Xmm::ERM)
      continue;	//	nothing from that thing
    const OdfInfo::ObservationExposureList &l =
    	odf->observationExposures(ins->id());

    /*
     * Initalize hash map for all the instruments with the exposure flag set to "no"
     */
    expoMap[Xmm::toString(ins->id())]  = "no";
    for (OdfInfo::ObservationExposureList::const_iterator exp =
    	l.begin(); exp!=l.end(); ++exp) {
    	const OdfInfo::ObservationExposure *se = *exp;

    	if (OdfBrowser::toModeType(se->mode()) != OdfBrowser::Engineering)
    	{
    		// Set the exposure flag to "yes"
    		expoMap[Xmm::toString(ins->id())]  = "yes";
    	}
    }
  }

  /*
   * Create OBSERVATION tag with info about all exposures
   */
  QDomElement obs = doc.createElement("OBSERVATION");
  tag.appendChild(obs);
  QDomElement *inf = new QDomElement();
  *inf =	  doc.createElement("PARAM");
  inf->setAttribute("id","analysisoption");
  inf->setAttribute("default","0:all");
  obs.appendChild(*inf);
  *inf =	  doc.createElement("PARAM");
  inf->setAttribute("id","epicsrc");
  inf->setAttribute("default","no");
  obs.appendChild(*inf);
  *inf =	  doc.createElement("PARAM");
  inf->setAttribute("id","ra");
  inf->setAttribute("default","-999");
  obs.appendChild(*inf);
  *inf =	  doc.createElement("PARAM");
  inf->setAttribute("id","dec");
  inf->setAttribute("default","-999");
  obs.appendChild(*inf);
  *inf =	  doc.createElement("PARAM");
  inf->setAttribute("id","sourcename");
  // Change all these characters +,-,(,),#,{,},[,] to _
  string val = odf->proposalData().targetName();
  string parsedName = replaceall(val, "(", "_");
  parsedName = replaceall(parsedName, ")", "_");
  parsedName = replaceall(parsedName," ","_");
  parsedName = replaceall(parsedName,"+","_");
  parsedName = replaceall(parsedName,"-","_");
  parsedName = replaceall(parsedName,"-","_");
  parsedName = replaceall(parsedName,"__","_");
  inf->setAttribute("default",parsedName.c_str());
  obs.appendChild(*inf);
  *inf =	  doc.createElement("PARAM");
  inf->setAttribute("id","obsid");
  stringstream str;
  str << odf->proposalId();
  str <<odf->observationId();
  string s = str.str();
  inf->setAttribute("default",s.c_str());
  obs.appendChild(*inf);

  *inf =	  doc.createElement("PARAM");
  inf->setAttribute("id","EPN");
  inf->setAttribute("default",expoMap["EPN"].c_str());
  obs.appendChild(*inf);

  *inf =	  doc.createElement("PARAM");
  inf->setAttribute("id","EMOS1");
  inf->setAttribute("default",expoMap["EMOS1"].c_str());
  obs.appendChild(*inf);

  *inf =	  doc.createElement("PARAM");
   inf->setAttribute("id","EMOS2");
   inf->setAttribute("default",expoMap["EMOS2"].c_str());
   obs.appendChild(*inf);

  *inf =	  doc.createElement("PARAM");
  inf->setAttribute("id","RGS1");
  inf->setAttribute("default",expoMap["RGS1"].c_str());
  obs.appendChild(*inf);

  *inf =	  doc.createElement("PARAM");
  inf->setAttribute("id","RGS2");
  inf->setAttribute("default",expoMap["RGS2"].c_str());
  obs.appendChild(*inf);

  *inf =	  doc.createElement("PARAM");
  inf->setAttribute("id","OM");
  inf->setAttribute("default",expoMap["OM"].c_str());
  obs.appendChild(*inf);

  *inf =	  doc.createElement("PARAM");
  inf->setAttribute("id","OM_sourcematch");
  inf->setAttribute("default","0.000277777");
  obs.appendChild(*inf);

  delete inf;


  for (unsigned i=0; i<xmmPayLoad.instruments().size(); ++i) {
    XmmInstrument *ins = xmmPayLoad.instruments()[i];

    if (ins->id() == Xmm::ERM)
      continue;	//	nothing from that thing

    errstr<<"Processing Instrument... " << Xmm::toString(ins->id()) << Sas::message(Msg::App, Msg::Verbose);

	QDomElement inst = doc.createElement("INSTRUMENT");
	string i = Xmm::toString(ins->id());
	QString qs(i.c_str());
	inst.setAttribute("value",qs);
	tag.appendChild(inst);
    const OdfInfo::ObservationExposureList &l =
      odf->observationExposures(ins->id());
    for (OdfInfo::ObservationExposureList::const_iterator exp =
	   l.begin(); exp!=l.end(); ++exp) {
      const OdfInfo::ObservationExposure *se = *exp;
      if (OdfBrowser::toModeType(se->mode()) != OdfBrowser::Engineering)
      {
    	  QDomElement exp = doc.createElement("EXPOSURE");
    	  string expid=se->id().substr(2,6);

    	  exp.setAttribute("expid",expid.c_str());
    	  if (Xmm::toString(se->instrument()) == "OM")
    		  exp.setAttribute("mode",Xmm::toString(se->dataMode()).c_str());
    	  else
    		  exp.setAttribute("mode",Xmm::toString(se->mode()).c_str());
    	  std::stringstream str;
    	  str << se->stop()-se->start();
    	  string d = str.str();
    	  exp.setAttribute("duration",d.c_str());
    	  exp.setAttribute("process","yes");
    	  inst.appendChild(exp);
    	  string confFile;

    	  string instExp;
    	  if (i == "EPN")
    	  {
    		  confFile="config/epn_confFile.xml";
    		  instExp ="PN";
    	  }
    	  else if (i == "EMOS1")
    	  {
    		  confFile = "config/emos1_confFile.xml";
    		  instExp ="M1";
    	  }
    	  else if (i == "EMOS2")
    	  {
    		  confFile = "config/emos2_confFile.xml";
    		  instExp ="M2";
    	  }
    	  else if (i == "RGS1")
    	  {
    		  confFile = "config/rgs1_confFile.xml";
    		  instExp ="R1";
    	  }
    	  else if (i == "RGS2")
    	  {
    		  confFile = "config/rgs2_confFile.xml";
    		  instExp ="R2";
    	  }
    	  else if (i == "OM")
    	  {
    		  confFile = "config/om_confFile.xml";
    		  instExp ="OM";
    	  }
    	  else
   		  errstr << "Unknown instrument conficuration file"
				  << Sas::error("UnknownInstrumentConfFile");

    	  SasFileLocator locator("");
		  string fileName = fileLocator.locate(confFile, "", false);


    	  /*
		   * Read xml file
		   */
		  QDomDocument xmlDoc("INSTRUMENT");
		  QFile file(fileName.c_str());
		  if( !file.open( IO_ReadOnly ) )
			  errstr << "Error reading instrument configuration file for: "
				  << i << Sas::error("ErrorOpeningInstConfFile");
		  if( !xmlDoc.setContent( &file ) )
		  {
			  file.close();
			  errstr << "Error parsing instrument configuration file for: "
				  << i << Sas::error("ErrorParsingInstConfFile");
		  }
		  file.close();
		  QDomElement  prod = xmlDoc.documentElement();

		  /*
		   * Add the PATTERN keyword depending on the MOS mode
		   */
		  // Default expression...

		  QDomNodeList nl = prod.elementsByTagName("TASK");
		  for (unsigned int ii = 0 ; ii < nl.length() ;++ii )
		  {
			  /*
			   * Add the instexpids values to epproc, emproc or rgsproc
			   */
			  QDomNodeList paramNL = nl.item(ii).childNodes();

			  for (unsigned int p = 0 ; p < paramNL.length() ; ++p)
			  {
				  if ( ( nl.item(ii).attributes().item(1).nodeValue() == "epproc" ) ||
						  ( nl.item(ii).attributes().item(1).nodeValue() == "emproc" ) )
				  {
					  QString expr = paramNL.item(p).attributes().item(1).nodeValue();

					  if ( paramNL.item(p).attributes().item(0).nodeValue() == "instexpids")
					  {
						  instExp = instExp+expid.c_str();
						  paramNL.item(p).attributes().item(1).setNodeValue(instExp.c_str());
					  }

					  if ( (Xmm::toString(se->mode()) == "FastBurst" ) &&
							  ( paramNL.item(p).attributes().item(0).nodeValue() == "burst")  )
					  {
						  paramNL.item(p).attributes().item(1).setNodeValue("yes");
					  }
				  }
				  /*
				   * GTI Filtering or Lightcurve
				   */
				  if ( ( nl.item(ii).attributes().item(0).nodeValue() == "clean_evtfile" )  &&
					( nl.item(ii).attributes().item(1).nodeValue() == "evselect" ) )
				  {
					  QDomNodeList paramNL = nl.item(ii).childNodes();

					  if  ( (i == "EMOS1") || (i == "EMOS2") )
					  {
						  QString expr = paramNL.item(p).attributes().item(1).nodeValue();

						  if ( paramNL.item(p).attributes().item(0).nodeValue() == "expression")
						  {
							  if ( (Xmm::toString(se->mode()) == "FastUncompressed" ) ||
									  (Xmm::toString(se->mode()) == "FastCompressed" ) )
							  {
								  if (expr.find("[2",expr.find("PI")) != -1)
								  {
									  expr = expr.replace(expr.find("[2",expr.find("PI")),2,"[5");
								  }
								  expr = expr +" && (PATTERN == 0)";
							  }
							  else
							  {
								  expr = expr +" && (PATTERN <= 12)";
							  }
						  }
						  paramNL.item(p).attributes().item(1).setNodeValue(expr);
					  }
				  }
			  }  // Param loop
			  /*
			   * Add the PATTERN keyword depending on the OM mode
			   */
			  if ( nl.item(ii).attributes().item(0).nodeValue() == "om_processing" )
			  {


				  if (Xmm::toString(se->dataMode()) == "Imaging")
				  {
					  nl.item(ii).attributes().item(1).setNodeValue("omichain");
				  }
				  else if(Xmm::toString(se->dataMode()) == "Fast")
				  {
					  nl.item(ii).attributes().item(1).setNodeValue("omfchain");
				  }
				  else
				  {
					  nl.item(ii).attributes().item(1).setNodeValue("omgchain");
				  }
			  }

		  } // Task loop






		  QDomNodeList n = prod.childNodes();
		  vector<QDomNode> vec;
		  for (unsigned int p = 0 ; p < n.length() ;)
		  {
			  QDomNode val = n.item(0);
			  exp.appendChild(val);
			  QDomNode tt = val.cloneNode(true);
			  vec.push_back(tt);

		  }
		  /*
		   * if EMOS1-2 are in Timing, we have to add an Imaging exposure with the same
		   * exposure ID
		   */
		  QDomElement exp2;
		  if( ( (i == "EMOS1") || (i == "EMOS2") )  &&  (
				  (Xmm::toString(se->mode()) == "FastUncompressed" ) || (Xmm::toString(se->mode()) == "FastCompressed" ) ) )
		  {
		   	  exp2 = doc.createElement("EXPOSURE");
	    	  exp2.setAttribute("expid",expid.c_str());
	    	  exp2.setAttribute("mode","PrimeFullWindow");
	    	  exp2.setAttribute("duration",d.c_str());
	    	  exp2.setAttribute("process","yes");
	    	  inst.appendChild(exp2);
			  for (unsigned int p = 0 ; p < vec.size() ; ++p)
			  {

				  for (unsigned int i = 0 ; i < vec[p].childNodes().length() ; ++i)
				  {
					  if ( ( vec[p].childNodes().item(i).attributes().item(0).nodeValue() == "clean_evtfile" )  &&
						( vec[p].childNodes().item(i).attributes().item(1).nodeValue() == "evselect" ) )
					  {
						  QDomNodeList paramNL = vec[p].childNodes().item(i).childNodes();

						  if ( paramNL.item(0).attributes().item(0).nodeValue() == "expression")
						  {
							  QString expr = paramNL.item(0).attributes().item(1).nodeValue();

							  if (expr.find("[5",expr.find("PI")) != -1)
							  {
								  expr = expr.replace(expr.find("[5",expr.find("PI")),2,"[2");
							  }

							  expr = expr.replace(expr.find("0",expr.find("PATTERN")),2,"12)");

							  paramNL.item(0).attributes().item(1).setNodeValue(expr);
						  }
					  }
				  }
				  exp2.appendChild(vec[p]);
			  }

		  }

      }
    }
  }


   QString xml = doc.toString();

   std::ofstream os(outputFileName.c_str());
   os << xml;



}
